@@ -7,13 +7,31 @@ from webob import Request, Response, exc
 from lxml import etree
 import unittest
 import doctest
+import httplib
+import socket
 import os
 
 import pyquery
 from pyquery import PyQuery as pq
 from ajax import PyQuery as pqa
 
+socket.setdefaulttimeout(1)
+
+try:
+    conn = httplib.HTTPConnection("pyquery.org:80")
+    conn.request("GET", "/")
+    response = conn.getresponse()
+except socket.timeout:
+    GOT_NET=False
+else:
+    GOT_NET=True
+
+def with_net(func):
+    if GOT_NET:
+        return func
+
 dirname = os.path.dirname(os.path.abspath(pyquery.__file__))
+docs = os.path.join(os.path.dirname(dirname), 'docs')
 path_to_html_file = os.path.join(dirname, 'test.html')
 
 def input_app(environ, start_response):
@@ -40,6 +58,14 @@ class TestReadme(doctest.DocFileCase):
         test = self._dt_test
         test.globs.update(globals())
 
+for filename in os.listdir(docs):
+    if filename.endswith('.txt'):
+        if not GOT_NET and filename in ('ajax.txt', 'tips.txt'):
+            continue
+        klass_name = 'Test%s' % filename.replace('.txt', '').title()
+        path = os.path.join(docs, filename)
+        exec '%s = type("%s", (TestReadme,), dict(path=path))' % (klass_name, klass_name)
+
 class TestTests(doctest.DocFileCase):
     path = os.path.join(dirname, 'tests.txt')
 
@@ -95,11 +121,29 @@ class TestSelector(unittest.TestCase):
                 <input name="radio" type="radio" value="one"/>
                 <input name="radio" type="radio" value="two" checked="checked"/>
                 <input name="radio" type="radio" value="three"/>
+                <input name="checkbox" type="checkbox" value="a"/>
+                <input name="checkbox" type="checkbox" value="b" checked="checked"/>
+                <input name="checkbox" type="checkbox" value="c"/>
+                <input name="button" type="button" value="button" />
+                <button>button</button>
               </form>
             </body>
            </html>
            """
 
+    html5 = """
+           <html>
+            <body>
+              <h1>Heading 1</h1>
+              <h2>Heading 2</h2>
+              <h3>Heading 3</h3>
+              <h4>Heading 4</h4>
+              <h5>Heading 5</h5>
+              <h6>Heading 6</h6>
+            </body>
+           </html>
+           """
+
     def test_selector_from_doc(self):
         doc = etree.fromstring(self.html)
         assert len(self.klass(doc)) == 1
@@ -142,10 +186,21 @@ class TestSelector(unittest.TestCase):
         #test on the form
         e = self.klass(self.html4)
         assert len(e(':disabled')) == 1
-        assert len(e('input:enabled')) == 5
+        assert len(e('input:enabled')) == 9
         assert len(e(':selected')) == 1
-        assert len(e(':checked')) == 1
+        assert len(e(':checked')) == 2
         assert len(e(':file')) == 1
+        assert len(e(':input')) == 12
+        assert len(e(':button')) == 2
+        assert len(e(':radio')) == 3
+        assert len(e(':checkbox')) == 3
+
+        #test on other elements
+        e = self.klass(self.html5)
+        assert len(e(":header")) == 6
+        assert len(e(":parent")) == 2
+        assert len(e(":empty")) == 6
+        assert len(e(":contains('Heading')")) == 6
 
 class TestTraversal(unittest.TestCase):
     klass = pq
@@ -175,6 +230,11 @@ class TestTraversal(unittest.TestCase):
         assert len(self.klass('#node2', self.html).find('span')) == 2
         assert len(self.klass('div', self.html).find('span')) == 3
 
+    def test_each(self):
+        doc = self.klass(self.html)
+        doc('span').each(lambda e: e.wrap("<em></em>"))
+        assert len(doc('em')) == 3
+
     def test_map(self):
         def ids_minus_one(i, elem):
             return int(self.klass(elem).attr('id')[-1]) - 1
@@ -184,6 +244,16 @@ class TestTraversal(unittest.TestCase):
         assert len(self.klass('div', self.html).find('span').end()) == 2
         assert len(self.klass('#node2', self.html).find('span').end()) == 1
 
+
+class TestOpener(unittest.TestCase):
+
+    def test_custom_opener(self):
+        def opener(url):
+            return '<html><body><div class="node"></div>'
+
+        doc = pq(url='http://example.com', opener=opener)
+        assert len(doc('.node')) == 1, doc
+
 def application(environ, start_response):
     req = Request(environ)
     response = Response()
@@ -201,6 +271,7 @@ def secure_application(environ, start_response):
 class TestAjaxSelector(TestSelector):
     klass = pqa
 
+    @with_net
     def test_proxy(self):
         e = self.klass([])
         val = e.get('http://pyquery.org/')
@@ -234,6 +305,22 @@ class TestAjaxSelector(TestSelector):
         val = n.post('/')
         assert len(val('a')) == 1, val
 
+class TestManipulating(unittest.TestCase):
+    html = '''
+    <div class="portlet">
+      <a href="/toto">Test<img src ="myimage" />My link text</a>
+      <a href="/toto2"><img src ="myimage2" />My link text 2</a>
+    </div>
+    '''
+
+    def test_remove(self):
+        d = pq(self.html)
+        d('img').remove()
+        val = d('a:first').html()
+        assert val == 'Test My link text', repr(val)
+        val = d('a:last').html()
+        assert val == ' My link text 2', repr(val)
+
 if __name__ == '__main__':
     fails, total = unittest.main()
     if fails == 0:
